package com.arms.api.analysis.time.service;

import com.arms.api.analysis.time.dto.TimeDTO;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.RangeQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.must.TermQueryMust;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.List;

@Slf4j
@Service("analysisTime")
public class AnalysisTimeImpl extends EsCommonRepositoryWrapper<AlmIssueEntity> implements AnalysisTime {

    @Override
    public List<AlmIssueEntity> findUpdatedIssues(TimeDTO timeDTO) {

        log.info("[지라이슈_서비스_프로세스 :: 업데이트_지라이슈_목록_조회] :: pdServiceLink = {}", timeDTO);

        LocalDate now = LocalDate.now(ZoneId.of("UTC"));
        LocalDate startDate = now.minusYears(1L);

        EsQuery esQuery = new EsQueryBuilder()
                .bool(  new TermQueryMust("pdServiceId", timeDTO.getPdServiceId()),
                        new TermsQueryFilter("pdServiceVersions", timeDTO.getPdServiceVersionLinks()),
                        RangeQueryFilter.of("updated").betweenDate(startDate.toString(),String.valueOf(now))
                );

        timeDTO.setEsQuery(esQuery);

        return findRecentHits(timeDTO).toDocs();
    }
}
