package com.arms.api.issue.almapi.model.entity;

import com.arms.egovframework.javaservice.esframework.annotation.ElasticSearchIndex;
import com.arms.egovframework.javaservice.esframework.annotation.ElasticSearchTemplateConfig;
import com.arms.egovframework.javaservice.esframework.annotation.Recent;
import com.arms.egovframework.javaservice.esframework.annotation.RollingIndexName;
import com.arms.egovframework.javaservice.esframework.model.entity.BaseEntity;
import com.fasterxml.jackson.annotation.*;
import lombok.*;
import org.springframework.data.annotation.Id;
import org.springframework.data.elasticsearch.annotations.Document;
import org.springframework.data.elasticsearch.annotations.Field;
import org.springframework.data.elasticsearch.annotations.FieldType;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.List;

@Data
@Builder
@AllArgsConstructor
@Getter
@Setter
@Document(indexName = "#{@jiraissue}", createIndex = false)
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.PROPERTY, property = "_class")
@JsonTypeName("com.arms.api.engine.models.지라이슈")
// @JsonTypeName("om.arms.api.index_entity.이슈_인덱스")
@JsonIgnoreProperties(ignoreUnknown = true)
@ElasticSearchTemplateConfig
@ElasticSearchIndex
@EqualsAndHashCode(exclude = {"recent","rawData"})
public class AlmIssueEntity implements BaseEntity {

    //////////////
    @Id
    @Field(type = FieldType.Keyword)
    private String id; // Elasticsearch의 문서 식별자

    @Recent
    @Field(type = FieldType.Boolean, name = "recent")
    private boolean recent;


    public AlmIssueEntity() {
    }

    //////////////
    public void generateId() {
        if (timestamp == null) {
            this.timestamp = new Date();
        }

        // 타임스탬프를 UTC+9로 변환
        LocalDateTime localDateTime = LocalDateTime.ofInstant(timestamp.toInstant(), ZoneId.of("UTC+9"));
        ZonedDateTime zonedDateTime = ZonedDateTime.of(localDateTime, ZoneId.of("UTC+9"));
        this.timestamp = Date.from(zonedDateTime.toInstant());

        this.id = this.jira_server_id + "_" + this.project.getKey() + "_" + this.key;
    }

    private Long jira_server_id;

    @Field(type = FieldType.Date, name = "@timestamp")
    private Date timestamp;

    @Field(type = FieldType.Auto, name = "issueID")
    private String issueID;

    @Field(type = FieldType.Keyword, name = "key")
    private String key;

    @Field(type = FieldType.Text, name = "self")
    private String self;

    @Field(type = FieldType.Keyword, name = "parentReqKey")
    private String parentReqKey;

    @Field(type = FieldType.Keyword, name = "upperKey")
    private String upperKey;

    @Field(type = FieldType.Boolean, name = "isReq")
    private Boolean isReq;

    @Field(type = FieldType.Auto, name = "etc")
    private Object etc;

    @Field(type = FieldType.Percolator, name = "queries")
    private List<String> percolatorQueries;

//    @Field(type = FieldType.Nested)
    private AlmIssueEntity.프로젝트 project;

//    @Field(type = FieldType.Nested)
    private AlmIssueEntity.이슈유형 issuetype;

//    @Field(type = FieldType.Nested)
    private AlmIssueEntity.생성자 creator;

//    @Field(type = FieldType.Nested)
    private AlmIssueEntity.보고자 reporter;

//    @Field(type = FieldType.Nested, fielddata = true)
    private AlmIssueEntity.담당자 assignee;

    @Field(type = FieldType.Text)
    private List<String> labels;

//    @Field(type = FieldType.Nested)
    private AlmIssueEntity.우선순위 priority;

//    @Field(type = FieldType.Nested, fielddata = true)
    private AlmIssueEntity.상태 status;

//    @Field(type = FieldType.Nested)
    private AlmIssueEntity.해결책 resolution;

    @Field(type = FieldType.Text, name = "resolutiondate")
    private String resolutiondate;

    @Field(type = FieldType.Date, name = "created")
    private String created;

    @Field(type = FieldType.Date, name = "updated")
    private String updated;


    private AlmIssueEntity.삭제 deleted;

//    @Field(type = FieldType.Nested)
    private List<AlmIssueEntity.워크로그> worklogs;

    @Field(type = FieldType.Integer, name = "timespent")
    private Integer timespent;

    @Field(type = FieldType.Text, name = "summary")
    private String summary;

    @Field(type = FieldType.Text, name = "pdServiceId", fielddata = true)
    private Long pdServiceId;

    @Field(type = FieldType.Long, name = "cReqLink")
    private Long cReqLink ;

    @Field(type = FieldType.Long, name = "pdServiceVersions")
    private List<Long> pdServiceVersions;

    @Field(type = FieldType.Keyword, name = "linkedIssues")
    private List<String> linkedIssues;

    private AlmIssueEntity.암스_요구사항_속성 cReqProperty;

    @JsonIgnore
    @Field(type = FieldType.Text, name = "rawData")
    private String rawData;


    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 암스_요구사항_속성 {
        @Field(type = FieldType.Long, name = "cReqPriorityLink")
        @JsonProperty("cReqPriorityLink")
        private Long cReqPriorityLink;

        @Field(type = FieldType.Keyword, name = "cReqPriorityName")
        @JsonProperty("cReqPriorityName")
        private String cReqPriorityName;

        @Field(type = FieldType.Long, name = "cReqDifficultyLink")
        @JsonProperty("cReqDifficultyLink")
        private Long cReqDifficultyLink;

        @Field(type = FieldType.Keyword, name = "cReqDifficultyName")
        @JsonProperty("cReqDifficultyName")
        private String cReqDifficultyName;

        @Field(type = FieldType.Long, name = "cReqStateLink")
        @JsonProperty("cReqStateLink")
        private Long cReqStateLink;

        @Field(type = FieldType.Keyword, name = "cReqStateName")
        @JsonProperty("cReqStateName")
        private String cReqStateName;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 삭제 {
        @Field(type = FieldType.Boolean, name = "deleted_isDeleted")
        @JsonProperty("deleted_isDeleted")
        private Boolean isDeleted;

        @Field(type = FieldType.Date, name = "deleted_date")
        @JsonProperty("deleted_date")
        private String deleted_date;
    }



    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 프로젝트 {
        @Field(type = FieldType.Text, name = "project_self")
        @JsonProperty("project_self")
        private String self;

        @Field(type = FieldType.Text, name = "project_id")
        @JsonProperty("project_id")
        private String id;

        @Field(type = FieldType.Text, name = "project_key")
        @JsonProperty("project_key")
        private String key;

        @Field(type = FieldType.Text, name = "project_name")
        @JsonProperty("project_name")
        private String name;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 이슈유형 {
        // 온프레미스, 클라우드 공통
        @Field(type = FieldType.Text, name = "issuetype_self")
        @JsonProperty("issuetype_self")
        private String self;

        @Field(type = FieldType.Text, name = "issuetype_id")
        @JsonProperty("issuetype_id")
        private String id;

        @Field(type = FieldType.Text, name = "issuetype_description")
        @JsonProperty("issuetype_description")
        private String description;

        @Field(type = FieldType.Text, name = "issuetype_name")
        @JsonProperty("issuetype_name")
        private String name;

        @Field(type = FieldType.Boolean, name = "issuetype_subtask")
        @JsonProperty("issuetype_subtask")
        private Boolean subtask;

        // 클라우드만 사용
        @Field(type = FieldType.Text, name = "issuetype_untranslatedName")
        @JsonProperty("issuetype_untranslatedName")
        private String untranslatedName;

        @Field(type = FieldType.Integer, name = "issuetype_hierarchyLevel")
        @JsonProperty("issuetype_hierarchyLevel")
        private Integer hierarchyLevel;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 생성자 {

        @Field(type = FieldType.Text, name = "creator_accountId")
        @JsonProperty("creator_accountId")
        private String accountId;

        @Field(type = FieldType.Text, name = "creator_emailAddress")
        @JsonProperty("creator_emailAddress")
        private String emailAddress;

        @Field(type = FieldType.Text, name = "creator_displayName")
        @JsonProperty("creator_displayName")
        private String displayName;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 보고자 {

        @Field(type = FieldType.Text, name = "reporter_accountId")
        @JsonProperty("reporter_accountId")
        private String accountId;

        @Field(type = FieldType.Text, name = "reporter_emailAddress")
        @JsonProperty("reporter_emailAddress")
        private String emailAddress;

        @Field(type = FieldType.Text, name = "reporter_displayName")
        @JsonProperty("reporter_displayName")
        private String displayName;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 담당자 {

        @Field(type = FieldType.Text, name = "assignee_accountId")
        @JsonProperty("assignee_accountId")
        private String accountId;

        @Field(type = FieldType.Text, name = "assignee_emailAddress", fielddata = true)
        @JsonProperty("assignee_emailAddress")
        private String emailAddress;

        @Field(type = FieldType.Text, name = "assignee_displayName")
        @JsonProperty("assignee_displayName")
        private String displayName;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 우선순위 {

        // 온프레미스, 클라우드 공통
        @Field(type = FieldType.Text, name = "priority_self")
        @JsonProperty("priority_self")
        private String self;

        @Field(type = FieldType.Text, name = "priority_id")
        @JsonProperty("priority_id")
        private String id;

        @Field(type = FieldType.Text, name = "priority_name")
        @JsonProperty("priority_name")
        private String name;

        @Field(type = FieldType.Text, name = "priority_description")
        @JsonProperty("priority_description")
        private String description;

        // 클라우드
        @Field(type = FieldType.Boolean, name = "priority_isDefault")
        @JsonProperty("priority_isDefault")
        private boolean isDefault;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 상태 {

        // 온프레미스, 클라우드 공통
        @Field(type = FieldType.Text, name = "status_self")
        @JsonProperty("status_self")
        private String self;

        @Field(type = FieldType.Text, name = "status_id")
        @JsonProperty("status_id")
        private String id;

        @Field(type = FieldType.Text, name = "status_name", fielddata = true)
        @JsonProperty("status_name")
        private String name;

        @Field(type = FieldType.Text, name = "status_description")
        @JsonProperty("status_description")
        private String description;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 해결책 {


        // 온프레미스, 클라우드 공통
        @Field(type = FieldType.Text, name = "resolution_self")
        @JsonProperty("resolution_self")
        private String self;

        @Field(type = FieldType.Text, name = "resolution_id")
        @JsonProperty("resolution_id")
        private String id;

        @Field(type = FieldType.Text, name = "resolution_name")
        @JsonProperty("resolution_name")
        private String name;

        @Field(type = FieldType.Text, name = "resolution_description")
        @JsonProperty("resolution_description")
        private String description;

        // 클라우드
        @Field(type = FieldType.Boolean, name = "resolution_isDefault")
        @JsonProperty("resolution_isDefault")
        private boolean isDefault;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 워크로그 {

        @Field(type = FieldType.Text, name = "worklogs_self")
        @JsonProperty("worklogs_self")
        private String self;

        @Field(type = FieldType.Nested)
        private 저자 author;

        @Field(type = FieldType.Nested)
        private 수정한_저자 updateAuthor;

        @Field(type = FieldType.Text, name = "worklogs_created")
        @JsonProperty("worklogs_created")
        private String created;

        @Field(type = FieldType.Text, name = "worklogs_updated")
        @JsonProperty("worklogs_updated")
        private String updated;

        @Field(type = FieldType.Text, name = "worklogs_started")
        @JsonProperty("worklogs_started")
        private String started;

        @Field(type = FieldType.Text, name = "worklogs_timeSpent")
        @JsonProperty("worklogs_timeSpent")
        private String timeSpent;

        @Field(type = FieldType.Integer, name = "worklogs_timeSpentSeconds")
        @JsonProperty("worklogs_timeSpentSeconds")
        private Integer timeSpentSeconds;

        @Field(type = FieldType.Text, name = "worklogs_id")
        @JsonProperty("worklogs_id")
        private String id;

        @Field(type = FieldType.Text, name = "worklogs_issueId")
        @JsonProperty("worklogs_issueId")
        private String issueId;
    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 저자 {

        @Field(type = FieldType.Text, name = "worklogs_author_accountId")
        @JsonProperty("worklogs_author_accountId")
        private String accountId;

        @Field(type = FieldType.Text, name = "worklogs_author_emailAddress")
        @JsonProperty("worklogs_author_emailAddress")
        private String emailAddress;

    }

    @Getter
    @Setter
    @Builder
    @ToString
    @NoArgsConstructor
    @AllArgsConstructor
    @JsonInclude(JsonInclude.Include.NON_NULL)
    @EqualsAndHashCode
    public static class 수정한_저자 {

        @Field(type = FieldType.Text, name = "worklogs_updateAuthor_accountId")
        @JsonProperty("worklogs_updateAuthor_accountId")
        private String accountId;

        @Field(type = FieldType.Text, name = "worklogs_updateAuthor_emailAddress")
        @JsonProperty("worklogs_updateAuthor_emailAddress")
        private String emailAddress;

    }

    @RollingIndexName
    public String localDate() {
        return String.valueOf(LocalDate.now());
    }

    public boolean izEmpty(){
        return id==null||id.isEmpty();
    }

    public boolean izNotEmpty(){
        return !this.izEmpty();
    }

    public String 조회조건_아이디(){
        return this.getJira_server_id() + "_" +
                this.getProject().getKey() + "_" +
                this.getKey();
    }



}
