package com.arms.api.issue.almapi.service;

import com.arms.api.issue.almapi.model.vo.지라이슈_데이터;
import com.arms.api.issue.almapi.model.dto.AlmIssueCreationDTO;
import com.arms.api.issue.almapi.model.dto.요구사항_데이터;
import com.arms.api.serverinfo.model.enums.서버유형_정보;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.serverinfo.service.서버정보_서비스;
import com.arms.egovframework.javaservice.esframework.erros.response.ErrorCode;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Consumer;

@Slf4j
@Service
@RequiredArgsConstructor
public class 이슈전략_호출 {

    private 서버정보_서비스 서버정보_서비스;
    private 클라우드_지라_이슈전략 클라우드_지라_이슈전략;
    private 온프레미스_지라_이슈전략 온프레미스_지라이슈_전략;
    private 온프레미스_레드마인_이슈전략 온프레미스_레드마인_이슈전략;

    @Autowired
    public 이슈전략_호출(서버정보_서비스 서버정보_서비스,
                    클라우드_지라_이슈전략 클라우드_지라_이슈전략,
                    온프레미스_지라_이슈전략 온프레미스_지라이슈_전략,
                    온프레미스_레드마인_이슈전략 온프레미스_레드마인_이슈전략) {

        this.서버정보_서비스 = 서버정보_서비스;
        this.클라우드_지라_이슈전략 = 클라우드_지라_이슈전략;
        this.온프레미스_지라이슈_전략 = 온프레미스_지라이슈_전략;
        this.온프레미스_레드마인_이슈전략 = 온프레미스_레드마인_이슈전략;
    }

    private 이슈전략_등록_및_실행 settingIssueStrategy(서버정보_데이터 서버정보) {

        if (서버정보 == null || StringUtils.isEmpty(서버정보.getType())) {
            log.error("지라이슈 전략 등록 Error: 서버정보_유형 {}", ErrorCode.서버유형_정보오류.getErrorMsg());
            throw new IllegalArgumentException("지라이슈 전략 등록 Error: 서버정보_유형 " + ErrorCode.서버유형_정보오류.getErrorMsg());
        }

        서버유형_정보 서버_유형 = 서버유형_정보.valueOf(서버정보.getType());

        이슈전략 이슈전략;
        if (서버_유형 == 서버유형_정보.클라우드) {
            이슈전략 = this.클라우드_지라_이슈전략;
        }
        else if (서버_유형 == 서버유형_정보.온프레미스) {
            이슈전략 = this.온프레미스_지라이슈_전략;
        }
        else if (서버_유형 == 서버유형_정보.레드마인_온프레미스) {
            이슈전략 = this.온프레미스_레드마인_이슈전략;
        }
        else {
            throw new IllegalArgumentException("이슈 전략 확인 Error: 허용하지 않는 서버정보_유형입니다. :: "+ 서버_유형+ " :: " + ErrorCode.서버유형_정보오류.getErrorMsg());
        }

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = new 이슈전략_등록_및_실행();
        이슈전략_등록_및_실행.이슈전략_등록(이슈전략);

        return 이슈전략_등록_및_실행;
    }

    public List<지라이슈_데이터> getIssueList(Long connectId, String projectKeyOrId) {

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(connectId);

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = this.settingIssueStrategy(서버정보);
        List<지라이슈_데이터> responseAlmIssueList = 이슈전략_등록_및_실행.getIssueList(서버정보, projectKeyOrId);

        return responseAlmIssueList;

    }

    public 지라이슈_데이터 createIssue(Long connectId, AlmIssueCreationDTO almIssueCreationDTO) {

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(connectId);

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = this.settingIssueStrategy(서버정보);
        지라이슈_데이터 createdIssueDTO = 이슈전략_등록_및_실행.createIssue(서버정보, almIssueCreationDTO);

        return createdIssueDTO;

    }

    public Map<String,Object> 이슈_수정하기(Long 연결_아이디, String 이슈_키_또는_아이디,
                                      AlmIssueCreationDTO almIssueCreationDTO) {

        if (연결_아이디 == null) {
            log.error("이슈 수정하기 Error: 연결_아이디 {}", ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("이슈 수정하기 Error: 연결_아이디 " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (이슈_키_또는_아이디==null || 이슈_키_또는_아이디.isEmpty()) {
            log.error("이슈 수정하기 Error 이슈_키_또는_아이디가 없습니다.");
            throw new IllegalArgumentException(ErrorCode.파라미터_NULL_오류.getErrorMsg() + " :: 이슈 수정하기 Error 이슈_키_또는_아이디가 없습니다.");
        }

        if (almIssueCreationDTO == null) {
            log.error("이슈 수정하기 Error 지라이슈생성_데이터가 없습니다.");
            throw new IllegalArgumentException("이슈 수정하기 Error 지라이슈생성_데이터가 없습니다.");
        }

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(연결_아이디);

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = this.settingIssueStrategy(서버정보);
        Map<String,Object> 이슈수정_결과
                = 이슈전략_등록_및_실행.이슈_수정하기(서버정보, 이슈_키_또는_아이디, almIssueCreationDTO);

        return 이슈수정_결과;

    }

    public Map<String,Object> 이슈_상태_변경하기(Long 연결_아이디, String 이슈_키_또는_아이디, String 상태_아이디) {

        if (연결_아이디 == null) {
            log.error("이슈 상태 변경하기 Error: 연결_아이디 {}" , ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("이슈 상태 변경하기 Error: 연결_아이디 " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (이슈_키_또는_아이디 == null || 이슈_키_또는_아이디.isEmpty()) {
            log.error("이슈 상태 변경하기 Error 이슈_키_또는_아이디가 없습니다.");
            throw new IllegalArgumentException(ErrorCode.파라미터_NULL_오류.getErrorMsg() + " :: 이슈 상태 변경하기 Error 이슈_키_또는_아이디가 없습니다.");
        }

        if (상태_아이디 == null || 상태_아이디.isEmpty()) {
            log.error("이슈 상태 변경하기 Error 상태 아이디가 없습니다.");
            throw new IllegalArgumentException("이슈 상태 변경하기 Error 상태 아이디가 없습니다.");
        }

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(연결_아이디);

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = this.settingIssueStrategy(서버정보);
        Map<String,Object> 상태변경_결과
                = 이슈전략_등록_및_실행.이슈_상태_변경하기(서버정보, 이슈_키_또는_아이디, 상태_아이디);

        return 상태변경_결과;

    }

    public Map<String,Object> 이슈_삭제하기(Long 연결_아이디, String 이슈_키_또는_아이디) {

        if (연결_아이디 == null) {
            log.error("이슈 삭제 라벨 처리하기 Error: 연결_아이디 {}" , ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("이슈 삭제 라벨 처리하기 Error: 연결_아이디 " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (이슈_키_또는_아이디 == null || 이슈_키_또는_아이디.isEmpty()) {
            log.error("이슈 삭제 라벨 처리하기 Error 이슈_키_또는_아이디 {}" , ErrorCode.파라미터_NULL_오류.getErrorMsg());
            throw new IllegalArgumentException("이슈 삭제 라벨 처리하기 Error 이슈_키_또는_아이디 " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
        }

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(연결_아이디);

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = this.settingIssueStrategy(서버정보);
        Map<String,Object> 이슈삭제라벨_결과
                = 이슈전략_등록_및_실행.이슈_삭제하기(서버정보, 이슈_키_또는_아이디);

        return 이슈삭제라벨_결과;

    }

    public 지라이슈_데이터 이슈_상세정보_가져오기(요구사항_데이터 요구사항_데이터) throws Exception {
        지라이슈_데이터 이슈_상세정보_가져오기 = this.이슈_상세정보_가져오기(요구사항_데이터.get지라서버_아이디(), 요구사항_데이터.get이슈_키());
        if(이슈_상세정보_가져오기!=null) {
            이슈_상세정보_가져오기.set서버아이디(요구사항_데이터.get지라서버_아이디());
        }
        return 이슈_상세정보_가져오기;
    }

    public 지라이슈_데이터 이슈_상세정보_가져오기(Long 연결_아이디, String 이슈_키_또는_아이디) throws Exception {

        if (연결_아이디 == null) {
            log.error("이슈 상세정보 가져오기 Error: 연결_아이디 {}" , ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("이슈 상세정보 가져오기 Error: 연결_아이디 " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (이슈_키_또는_아이디 == null || 이슈_키_또는_아이디.isEmpty()) {
            log.error("이슈 상세정보 가져오기 Error 이슈_키_또는_아이디 {}" , ErrorCode.파라미터_NULL_오류.getErrorMsg());
            throw new IllegalArgumentException("이슈 상세정보 가져오기 Error 이슈_키_또는_아이디 " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
        }

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(연결_아이디);

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = this.settingIssueStrategy(서버정보);
        지라이슈_데이터 반환할_지라이슈_데이터
                = 이슈전략_등록_및_실행.이슈_상세정보_가져오기(서버정보, 이슈_키_또는_아이디);

        return 반환할_지라이슈_데이터;

    }

    public List<지라이슈_데이터> 연결이슈_가져오기(요구사항_데이터 요구사항_데이터) throws Exception {
        List<지라이슈_데이터> 연결이슈_가져오기 = this.연결이슈_가져오기(요구사항_데이터.get지라서버_아이디(), 요구사항_데이터.get이슈_키());
        List<지라이슈_데이터> 지라이슈_데이터들 = Optional.ofNullable(연결이슈_가져오기).orElse(new ArrayList<>());
        지라이슈_데이터들.forEach(서버아이디_추가(요구사항_데이터));
        return 지라이슈_데이터들;
    }

    public List<지라이슈_데이터> 연결이슈_가져오기(Long 연결_아이디, String 이슈_키_또는_아이디) throws Exception {

        if (연결_아이디 == null) {
            log.error("이슈 링크 가져오기 Error: 연결_아이디 {}" , ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("이슈 링크 가져오기 Error: 연결_아이디 " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (이슈_키_또는_아이디 == null || 이슈_키_또는_아이디.isEmpty()) {
            log.error("이슈 링크 가져오기 Error 이슈_키_또는_아이디 {}" , ErrorCode.파라미터_NULL_오류.getErrorMsg());
            throw new IllegalArgumentException("이슈 링크 가져오기 Error 이슈_키_또는_아이디 " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
        }

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(연결_아이디);

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = this.settingIssueStrategy(서버정보);
        List<지라이슈_데이터> 반환할_연결이슈_목록 = 이슈전략_등록_및_실행.연결이슈_가져오기(서버정보, 이슈_키_또는_아이디);

        return 반환할_연결이슈_목록;

    }

    public List<지라이슈_데이터> 서브테스크_가져오기(요구사항_데이터 요구사항_데이터) throws Exception {
        List<지라이슈_데이터> 서브테스크_가져오기
            = this.서브테스크_가져오기(요구사항_데이터.get지라서버_아이디(), 요구사항_데이터.get이슈_키());
        List<지라이슈_데이터> 지라이슈_데이터들
            = Optional.ofNullable(서브테스크_가져오기).orElse(new ArrayList<>());
        지라이슈_데이터들.forEach(서버아이디_추가(요구사항_데이터));
        return 지라이슈_데이터들;
    }

    private Consumer<지라이슈_데이터> 서버아이디_추가(요구사항_데이터 요구사항_데이터) {
        return a -> a.set서버아이디(요구사항_데이터.get지라서버_아이디());
    }

    public List<지라이슈_데이터> 서브테스크_가져오기(Long 연결_아이디, String 이슈_키_또는_아이디) throws Exception {

        if (연결_아이디 == null) {
            log.error("이슈 서브테스크 가져오기 Error: 연결_아이디 {}" , ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("이슈 서브테스크 가져오기 Error: 연결_아이디 " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (이슈_키_또는_아이디 == null || 이슈_키_또는_아이디.isEmpty()) {
            log.error("이슈 서브테스크 가져오기 Error 이슈_키_또는_아이디 {}" , ErrorCode.파라미터_NULL_오류.getErrorMsg());
            throw new IllegalArgumentException("이슈 서브테스크 가져오기 Error 이슈_키_또는_아이디 " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
        }

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(연결_아이디);

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = this.settingIssueStrategy(서버정보);
        List<지라이슈_데이터> 반환할_연결이슈_목록
                = 이슈전략_등록_및_실행.서브테스크_가져오기(서버정보, 이슈_키_또는_아이디);

        return 반환할_연결이슈_목록;
    }


    public List<지라이슈_데이터> 증분서브테스크_가져오기(요구사항_데이터 요구사항_데이터) {
        List<지라이슈_데이터> 증분서브테스크_가져오기
            = 증분서브테스크_가져오기(요구사항_데이터.get지라서버_아이디(), 요구사항_데이터.get이슈_키(), 요구사항_데이터.getStartDate(), 요구사항_데이터.getEndDate());
        return Optional.ofNullable(증분서브테스크_가져오기).orElse(new ArrayList<>());
    }

    public List<지라이슈_데이터> 증분서브테스크_가져오기(Long 연결_아이디, String 이슈_키_또는_아이디, String startDate, String endDate) {

        if (연결_아이디 == null) {
            log.error("이슈 서브테스크 가져오기 Error: 연결_아이디 {}" , ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
            throw new IllegalArgumentException("이슈 서브테스크 가져오기 Error: 연결_아이디 " + ErrorCode.파라미터_서버_아이디_없음.getErrorMsg());
        }

        if (이슈_키_또는_아이디 == null || 이슈_키_또는_아이디.isEmpty()) {
            log.error("이슈 서브테스크 가져오기 Error 이슈_키_또는_아이디 {}", ErrorCode.파라미터_NULL_오류.getErrorMsg());
            throw new IllegalArgumentException("이슈 서브테스크 가져오기 Error 이슈_키_또는_아이디 " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
        }

        서버정보_데이터 서버정보 = 서버정보_서비스.서버정보_검증(연결_아이디);

        이슈전략_등록_및_실행 이슈전략_등록_및_실행 = this.settingIssueStrategy(서버정보);
        List<지라이슈_데이터> 반환할_연결이슈_목록
            = 이슈전략_등록_및_실행.증분서브테스크_가져오기(서버정보, 이슈_키_또는_아이디,startDate,endDate);

        return 반환할_연결이슈_목록;
    }
}
