package com.arms.api.issue.priority.strategy;

import com.arms.api.issue.priority.model.IssuePriorityDTO;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.지라API_정보;
import com.arms.api.util.alm.지라유틸;
import com.arms.api.util.errors.ErrorLogUtil;
import lombok.AllArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.List;

@Component
@AllArgsConstructor
public class CloudJiraIssuePriorityStrategy implements IssuePriorityStrategy {

    private final 지라유틸 지라유틸;
    private final 지라API_정보 지라API_정보;

    @Override
    public List<IssuePriorityDTO> getIssuePriorityList(서버정보_데이터 서버정보) {

        WebClient webClient = 지라유틸.클라우드_통신기_생성(서버정보.getUri(), 서버정보.getUserId(), 서버정보.getPasswordOrToken());
        String priorityEndpoint = 지라API_정보.getEndpoint().getIssue().getPriority();

        try {
            List<IssuePriorityDTO> issuePriorityList = 지라유틸.get(webClient, priorityEndpoint,
                    new ParameterizedTypeReference<List<IssuePriorityDTO>>() {}).block();

            return issuePriorityList;
        }
        catch (Exception e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getIssuePriorityList Error", 서버정보.getType(), 서버정보.getUri()));
            throw new IllegalArgumentException(errorMessage);
        }
    }
}
