package com.arms.api.issue.priority.strategy;

import com.arms.api.issue.priority.model.OnpremiseRedmineIssuePriorityDTO;
import com.arms.api.issue.priority.model.IssuePriorityDTO;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.레드마인API_정보;
import com.arms.api.util.alm.레드마인유틸;
import com.arms.api.util.errors.ErrorLogUtil;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Component
@AllArgsConstructor
public class OnpremiseRedmineIssuePriorityStrategy implements IssuePriorityStrategy {

    private final 레드마인유틸 레드마인유틸;
    private final 레드마인API_정보 레드마인API_정보;

    @Override
    public List<IssuePriorityDTO> getIssuePriorityList(서버정보_데이터 서버정보) {

        WebClient webClient = 레드마인유틸.레드마인_웹클라이언트_통신기_생성(서버정보.getUri(), 서버정보.getPasswordOrToken());
        String endpoint = 레드마인API_정보.getEndpoint().getPriorityList();

        try {
            OnpremiseRedmineIssuePriorityDTO redmineIssuePriorityDTO
                    = 레드마인유틸.get(webClient, endpoint, OnpremiseRedmineIssuePriorityDTO.class).block();

            return Optional.ofNullable(redmineIssuePriorityDTO)
                    .map(OnpremiseRedmineIssuePriorityDTO::getIssue_priorities)
                    .orElseGet(Collections::emptyList)
                    .stream()
                    .filter(Objects::nonNull)
                    .filter(OnpremiseRedmineIssuePriorityDTO.IssuePriority::isActive)
                    .map(issuePriority -> convertIssuePriorityDTO(issuePriority, 서버정보.getUri()))
                    .collect(Collectors.toList());
        }
        catch (Exception e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getIssuePriorityList Error",
                            서버정보.getType(), 서버정보.getUri()));
            throw new IllegalArgumentException(errorMessage);
        }
    }

    private IssuePriorityDTO convertIssuePriorityDTO(OnpremiseRedmineIssuePriorityDTO.IssuePriority 우선순위, String 서버정보경로) {
        IssuePriorityDTO IssuePriorityDTO = new IssuePriorityDTO();

        IssuePriorityDTO.setId(String.valueOf(우선순위.getId()));
        IssuePriorityDTO.setName(우선순위.getName());
        IssuePriorityDTO.setDefault(우선순위.is_default());
        IssuePriorityDTO.setSelf(레드마인유틸.서버정보경로_체크(서버정보경로) + 레드마인API_정보.아이디_대체하기(레드마인API_정보.getEndpoint().getPriority(), String.valueOf(우선순위.getId())));

        return IssuePriorityDTO;
    }
}
