package com.arms.api.issue.resolution.strategy;

import com.arms.api.issue.resolution.model.이슈해결책_데이터;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.지라API_정보;
import com.arms.api.util.alm.지라유틸;
import com.arms.api.util.errors.ErrorLogUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.List;

@Component
public class 클라우드_지라_이슈해결책_전략 implements 이슈해결책_전략 {

    private final Logger 로그 = LoggerFactory.getLogger(this.getClass());

    private 지라유틸 지라유틸;
    private 지라API_정보 지라API_정보;

    @Autowired
    public 클라우드_지라_이슈해결책_전략(지라유틸 지라유틸,
                                    지라API_정보 지라API_정보) {
        this.지라유틸 = 지라유틸;
        this.지라API_정보 = 지라API_정보;
    }

    @Override
    public List<이슈해결책_데이터> 이슈해결책_목록_가져오기(서버정보_데이터 서버정보) {

        try {
            WebClient webClient = 지라유틸.클라우드_통신기_생성(서버정보.getUri(), 서버정보.getUserId(),
                                                            서버정보.getPasswordOrToken());
            String resolutionEndpoint = 지라API_정보.getEndpoint().getIssue().getResolution();
            List<이슈해결책_데이터> 반환할_이슈해결책_데이터_목록 = 지라유틸.get(webClient, resolutionEndpoint,
                                                    new ParameterizedTypeReference<List<이슈해결책_데이터>>() {}).block();

            return 반환할_이슈해결책_데이터_목록;
        }
        catch (Exception e) {
            String 에러로그 = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    "클라우드 지라("+ 서버정보.getConnectId() +") :: 이슈해결책_목록_가져오기");
            throw new IllegalArgumentException(에러로그);
        }
    }

}
