package com.arms.api.issue.status.strategy;

import com.arms.api.issue.status.vo.CloudJiraByIssueTypeIssueStatusVO;
import com.arms.api.issue.status.vo.IssueStatusVO;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.지라유틸;
import com.arms.api.util.errors.ErrorLogUtil;
import com.atlassian.jira.rest.client.api.JiraRestClient;
import com.atlassian.jira.rest.client.api.domain.Status;
import io.atlassian.util.concurrent.Promise;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

@Component
@AllArgsConstructor
public class OnpremiseJiraIssueStatusStrategy implements IssueStatusStrategy {

    private final 지라유틸 지라유틸;

    @Override
    public List<IssueStatusVO> getIssueStatusList(서버정보_데이터 서버정보) {

        JiraRestClient restClient = 지라유틸.온프레미스_통신기_생성(서버정보.getUri(),
                서버정보.getUserId(),
                서버정보.getPasswordOrToken());

        try {
            Promise<Iterable<Status>> statusesPromise = restClient.getMetadataClient().getStatuses();
            Iterable<Status> statuses = statusesPromise.claim();

            return StreamSupport.stream(statuses.spliterator(), false)
                    .map(this::convertIssueStatusVO)
                    .collect(Collectors.toList());
        }
        catch (Exception e) {
            String errorMessage =  ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getIssueStatusList Error", 서버정보.getType(), 서버정보.getUri()));
            throw new IllegalArgumentException(errorMessage);
        }
    }

    @Override
    public List<CloudJiraByIssueTypeIssueStatusVO> getIssueStatusListByProject(서버정보_데이터 서버정보,
                                                                               String projectId) {
        return null;
    }

    private IssueStatusVO convertIssueStatusVO(Status issueStatus) {
        return IssueStatusVO.builder()
                .self(issueStatus.getSelf().toString())
                .id(issueStatus.getId().toString())
                .name(issueStatus.getName())
                .description(issueStatus.getDescription()).build();
    }
}
