package com.arms.api.project.strategy;

import com.arms.api.project.vo.ProjectVO;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.지라유틸;
import com.arms.api.util.errors.ErrorLogUtil;
import com.atlassian.jira.rest.client.api.JiraRestClient;
import com.atlassian.jira.rest.client.api.domain.BasicProject;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

@Component
@AllArgsConstructor
public class OnpremiseJiraProjectStrategy implements ProjectStrategy {

    private final 지라유틸 지라유틸;

    @Override
    public ProjectVO getProject(서버정보_데이터 서버정보, String projectKeyOrId) {

        JiraRestClient restClient = 지라유틸.온프레미스_통신기_생성(서버정보.getUri(),
                서버정보.getUserId(),
                서버정보.getPasswordOrToken());

        try {
            BasicProject basicProject = restClient.getProjectClient()
                    .getProject(projectKeyOrId)
                    .claim();

            return this.convertProjectVO(basicProject);
        }
        catch (Exception e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getProject Error 프로젝트[%s] ",
                            서버정보.getType(), 서버정보.getUri(), projectKeyOrId));
            throw new IllegalArgumentException(errorMessage);
        }
    }

    @Override
    public List<ProjectVO> getProjectList(서버정보_데이터 서버정보) {

        JiraRestClient restClient = 지라유틸.온프레미스_통신기_생성(서버정보.getUri(),
                서버정보.getUserId(),
                서버정보.getPasswordOrToken());

        try {
            Iterable<BasicProject> basicProjectList = restClient.getProjectClient()
                    .getAllProjects()
                    .claim();

            return StreamSupport.stream(basicProjectList.spliterator(), false)
                    .map(this::convertProjectVO)
                    .collect(Collectors.toList());
        }
        catch (Exception e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getProjectList Error", 서버정보.getType(), 서버정보.getUri()));
            throw new IllegalArgumentException(errorMessage);
        }
    }

    private ProjectVO convertProjectVO(BasicProject project) {
        return ProjectVO.builder()
                .id(project.getId().toString())
                .key(project.getKey())
                .name(project.getName())
                .self(project.getSelf().toString()).build();
    }
}