package com.arms.egovframework.javaservice.esframework.model.vo.v2;

import static java.util.stream.Collectors.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.stream.Stream;

import lombok.Getter;
import org.elasticsearch.search.aggregations.Aggregation;
import org.elasticsearch.search.aggregations.Aggregations;
import org.elasticsearch.search.aggregations.bucket.MultiBucketsAggregation;
import org.springframework.data.elasticsearch.core.SearchHits;

import com.arms.egovframework.javaservice.esframework.model.vo.SearchResult;


public class DocumentAggregations implements SearchResult {

	@Getter
	private final Long totalHits;

	@Getter
	private final Long docSumCount;

	private final List<DocumentBucket> documentBuckets;

	public DocumentAggregations(SearchHits<?> searchHits) {
		this.totalHits = searchHits.getTotalHits();

		Map<String, Aggregation> mainBucketsAggregationMap = this.getTerm(searchHits.getAggregations())
				.entrySet()
				.stream()
				.collect(toMap(Entry::getKey, Entry::getValue));

		this.documentBuckets
			= new DocumentAggregations(mainBucketsAggregationMap).docBuckets();

		this.docSumCount = documentBuckets.stream().mapToLong(DocumentBucket::getDocCount).reduce(0, Long::sum);

	}

	public DocumentAggregations(List<DocumentBucket> documentBuckets) {
		this.totalHits = -1L;
		this.docSumCount = -1L;
		this.documentBuckets = documentBuckets;
	}

	DocumentAggregations(Map<String, Aggregation> bucketsAggregationMap) {

		this.documentBuckets = bucketsAggregationMap.entrySet()
			.stream()
			.flatMap(bucket->this.createDocumentBucket(bucket)).collect(toList());
		this.totalHits = -1L;
		this.docSumCount = -1L;
	}

	private Stream<DocumentBucket> createDocumentBucket(Entry<String, Aggregation> bucket) {

		return ((MultiBucketsAggregation) bucket.getValue())
				.getBuckets()
				.stream().map(bucketValue -> new DocumentBucket(bucket.getKey(),bucketValue));

	}

	private Map<String, Aggregation> getTerm(Aggregations aggregationsContainer){
		return Optional.ofNullable(aggregationsContainer).map(Aggregations::getAsMap).orElseGet(
			HashMap::new);
	}


	public List<DocumentBucket> deepestSubList() {
		return documentBuckets.stream()
			.flatMap(a -> a.getDeepestSubDocument().stream())
			.collect(toList());
	}


	public List<DocumentBucket> docBuckets() {
		return documentBuckets;
	}



}
