package com.arms.egovframework.javaservice.esframework.repository;

import static com.arms.config.ApplicationContextProvider.*;

import com.arms.egovframework.javaservice.esframework.factory.creator.기본_쿼리_생성기;
import com.arms.egovframework.javaservice.esframework.factory.creator.중첩_집계_쿼리_생성기;
import com.arms.egovframework.javaservice.esframework.model.dto.기본_검색_요청;
import com.arms.egovframework.javaservice.esframework.model.dto.기본_검색_집계_하위_요청;
import com.arms.egovframework.javaservice.esframework.model.entity.BaseEntity;
import com.arms.egovframework.javaservice.esframework.model.vo.DocumentSearchResult;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;

import lombok.extern.slf4j.Slf4j;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class EsCommonRepositoryWrapper<E extends BaseEntity> {

    @Autowired
    private FindRepository findRepository;

    private FindRepository ifNullGetBean(FindRepository findRepository){
        if(findRepository==null){
            return getBean(FindRepository.class);
        }
        return findRepository;
    }

    protected  <T extends 기본_검색_집계_하위_요청> DocumentAggregations aggregateRecentDocs(T requestDTO) {
        공통저장소<?, Serializable> repositoryByClass = ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.aggregateRecentDocs(중첩_집계_쿼리_생성기.포괄V2(requestDTO).생성(), null);
    }

    protected <T extends 기본_검색_집계_하위_요청>  DocumentAggregations aggregateDocs(T requestDTO) {
        공통저장소<?, Serializable> repositoryByClass = ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.aggregateDocs(중첩_집계_쿼리_생성기.포괄V2(requestDTO).생성(), null);
    }

    protected <T extends 기본_검색_집계_하위_요청> DocumentAggregations aggregateRecentDocsByDay(T requestDTO) {
        공통저장소<?, Serializable> repositoryByClass = ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.aggregateRecentDocs(중첩_집계_쿼리_생성기.포괄_일별(requestDTO).생성(), null);
    }

    protected <T extends 기본_검색_집계_하위_요청>  DocumentAggregations aggregateDocsByWeek(T requestDTO) {
        공통저장소<?, Serializable> repositoryByClass = ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.aggregateDocs(중첩_집계_쿼리_생성기.포괄_주별(requestDTO).생성(), null);
    }

    protected <T extends 기본_검색_요청> DocumentSearchResult<E> findRecentHits(T requestDTO) {
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.findRecentHits(기본_쿼리_생성기.기본검색(requestDTO.getEsQuery()).생성());
    }

    protected <T extends 기본_검색_요청> DocumentSearchResult<E> findHits(T requestDTO){
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.findHits(기본_쿼리_생성기.기본검색(requestDTO.getEsQuery()).생성());
    }

    protected E findRecentDocById(Serializable serializable) {
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.findRecentDocById(serializable);
    }

    protected E findDocById(Long id) {
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.findDocById(id);
    }

    protected Set<String> findIndexNamesByAlias(String indexName){
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.findIndexNamesByAlias(IndexCoordinates.of(indexName));
    }

    protected boolean deleteIndex(String indexName){
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.deleteIndex(IndexCoordinates.of(indexName));
    }

    protected E save(E entity){
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.save(entity);
    }

    protected E save(E entity,String indexName){
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());
        return repositoryByClass.save(entity,indexName);
    }

    protected String delete(E entity) {
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());

        return repositoryByClass.deleteDocByEntity(entity);
    }

    protected List<E> findAllDocs() {
        공통저장소<E, Serializable> repositoryByClass
            = (공통저장소<E, Serializable>)ifNullGetBean(findRepository).findRepositoryByClass(this.getGenericClass());

        return repositoryByClass.findAllDocs();
    }


    private Class<?> getGenericClass(){
        Type genericSuperclass = this.getClass().getGenericSuperclass();
        String typeName = ((ParameterizedType)genericSuperclass).getActualTypeArguments()[0].getTypeName();
        try{
            return Class.forName(typeName);
        }catch (ClassNotFoundException classNotFoundException){
            throw new IllegalArgumentException(classNotFoundException.getMessage());
        }
    }

}
